#! /usr/bin/env python
# -*- coding: utf-8 -*-

### 台本テキストを読み込んで yukktalk 形式テキストに変換するスクリプト
### Python2.7 で動作確認
### 
### 対応書式　鍵括弧と丸括弧とコロン
### 名前「台詞」
### 名前（台詞）
### 名前：台詞
### 以上の書式で書かれたテキストを yukktalk 形式テキストに変換
### 

import sys
import os.path
import datetime

import re
import codecs

# MeCab モジュールがあるならインポート
try :
	import MeCab
	isMeCab = True
except ImportError:
	isMeCab = False

# 最初にインタプリタ環境を初期化。あんまりスマートじゃないやり方
reload(sys)
sys.setdefaultencoding("utf-8")
# 日本語のファイルを読み込む設定
sys.stdout = codecs.getwriter("utf_8")(sys.stdout)
sys.stdin = codecs.getreader("utf_8")(sys.stdin)

#--------軽めな設定--------------------------------------------#
# 入力ファイル
# コマンドライン引数でファイル名を指定されなければこのファイルを読み込む
FILE = "test.txt"

# 字幕に発言者の名前を表示する
SHOW_NAME_TO_SUBTITLE = True
# 字幕表示一行あたりの最大文字数
MAX_LINE_LENGTH = 30
# 字幕の折り返しに禁則処理を行う
USE_LINE_BREAKING_RULE = True
# 禁則処理を厳密に行う
STRICT_LINE_BREAKING_RULE = True

# 棒読みちゃん用のお助け変換を行う
USE_BOYOMICHAN_HELP = True
# MeCab を使って漢字をひらく
USE_MECAB  = True
# 「は」を確実に読ませるために「ha」に変換
TRANS_HA = True
# 棒読みちゃん用台本の語尾から読点を取り去る
DELETE_GOBI_DAKUTEN = True

# 外側（内側）「台詞」　　な時どうするか
# 名前が括弧付きの場合、発言者指定も括弧つきでそのまま行う
SPEAKER_COMMAND_WITH_INSIDE_BRACKET = False
# ↑が True だった時。
# ◆外側（内側）　　となる
# ↑が False だった時、さらに設定。
# 括弧の内側か外側か、どっちを発言者指定にするか
NAME_IN_BRACKET_OUTSIDE_OR_INSIDE = u"INSIDE"
# ↑は括弧の "INSIDE" か "OUTSIDE" で指定
# "INSIDE" なら
# ◆内側
# "OUTSIDE" なら
# ◆外側



#---------細かい設定いろいろ------------------------------------------#
# 台詞枠に合わせる時などで使用
# 字幕の名前表示の前に全角空白をいれる数
ZENKAKU_SPASE_BEFORE_NAME = 0
# 字幕の名前表示の後に空改行をいれる数
NEWLINE_AFTER_NAME = 0
# 字幕の名前表示のフォントサイズ
# pt数で指定　0以下で本文と同じサイズ
NAME_SIZE = 0


# 名前と判断する最大の文字数（コマンド含む）
LONGEST_NAME_LENGTH = 40


#------------------------------------------------------------------------#
#---------文字列置換用のリスト--------------------------------------------#
# リストのリストで定義 (変換前のもの), (変換後のもの)
# 棒読みちゃん用のもの
BOYOMICHAN_TRANS_LIST=(
	# 置換の順番に注意　(変換前), (変換後)
	# 変換後の読みはカタカナで表記

	# MeCab でも直せないもの
	(u"雀卓", u"ジャンタク"),
	(u"良い", u"イイ"),
	(u"お腹", u"オナカ"),

	# 文字そのままだと聞こえが悪い表記の修正
	(u"チッ", u"チィ"),
	(u"ん？", u"ムゥ？"),
	
	# アルファベット読みの間違いの修正
	(u"−", u"マイナス"),
	(u"％", u"パーセント"),
	(u"ＫＮＤ", u"ケーエヌディー"),
	(u"ＯＰ", u"オープニング"),
	(u"ＥＤ", u"エンディング"),
	(u"ＧＭ", u"ジーエム"),
	(u"ＫＰ", u"キーパー"),
	(u"ＰＬ", u"プレイヤー"),
	(u"ＴＲＰＧ", u"ティーアールピージー"),
	(u"ＳＡＮ値", u"サンチ"),
	(u"ＳＡＮ", u"サン"),
	(u"ＩＮＴ", u"チセイ"),
	(u"ＣＯＮ", u"タイリョク"),
	(u"ＰＯＷ", u"セイシン"),
	(u"ＳＴＲ", u"キンリョク"),
	(u"ＥＤＵ", u"キョウヨウ"),
	(u"ＤＥＸ", u"ビンショウ"),
	(u"ＡＰＰ", u"アピアランス"),
	(u"ＳＩＺ", u"サイズ"),
	(u"Ｄ", u"ディー"),
	
	# 約物の修正
	(u"　", u""),
	(u"〈", u""),
	(u"〉", u""),
	(u"『", u"・"),
	(u"』", u"・"),
	(u"……", u"…"),
	
	# ダイスロール時間の修正
	(u"（コロコロ）", u"……"),

	# 全角数字の半角化は別の方法で
)
# もし MeCab モジュールが無いなら
if not isMeCab:
	# リストのリストで定義 (変換前のもの), (変換後のもの)
	# 棒読みちゃん用のもの
	BOYOMICHAN_TRANS_LIST=(
		# 置換の順番に注意　(変換前), (変換後)
		# 変換後の読みはカタカナで表記
		# 一話などでの間違いの修正
		(u"先生", u"センセイ"),
		(u"雀卓", u"ジャンタク"),
		(u"如何", u"イカガ"),
		(u"これは", u"これワ"),
		(u"一ヶ所", u"イッカショ"),
		(u"ワイン畑", u"ワインバタケ"),
		(u"血涙", u"ケツルイ"),
		(u"夕方頃", u"ユウガタゴロ"),
		(u"お三方", u"おサンカタ"),
		(u"お二方", u"おフタカタ"),
		(u"屑", u"クズ"),
		(u"閃い", u"ヒラメい"),
		
	
		# 送り仮名によって変わる読み間違いの修正
		(u"他所", u"ヨソ"),
		(u"他人", u"タニン"),
		(u"他", u"ホカ"),
		(u"お方", u"おカタ"),
		(u"方", u"ホウ"),
		(u"開け", u"アけ"),
		(u"開く", u"ヒラく"),
		(u"翻訳", u"ホンヤク"),
		(u"訳", u"ワケ"),
		(u"後ほど", u"ノチほど"),
		(u"後輩", u"コウハイ"),
		(u"後", u"アト"),
		(u"外国", u"ガイコク"),
		(u"外壁", u"ガイヘキ"),
		(u"予想外", u"ヨソウガイ"),
		(u"意外", u"イガイ"),
		(u"以外", u"イガイ"),
		(u"外", u"ソト"),
		(u"使用人", u"シヨウニン"),
		(u"一人", u"ヒトリ"),
		(u"二人", u"フタリ"),
		(u"図書館", u"トショカン"),
		# (u"人", u"ヒト"),
		
		# 指摘された間違いの修正
		(u"館", u"ヤカタ"),
		(u"頭", u"アタマ"),
			) + BOYOMICHAN_TRANS_LIST

# MeCab で品詞分解後に手動で置換
# これは単語の完全一致でのみ置換
MECAB_DIVIDE_REPLACE=(
	(u"後", u"アト"),
	(u"他", u"ホカ"),
	(u"方", u"カタ"),
	(u"間", u"アイダ"),
	(u"外", u"ソト"),
	(u"訳", u"ワケ"),
	(u"開け", u"アけ"),
	(u"閃い", u"ヒラメい"),
)


# 字幕用
# Aviutl の字幕用制御文字を入れていく
SUBTITLE_CTRL_TRANS_LIST=(
	# 置換の順番に注意　(変換前), (変換後)

	# ダイスロール後の字幕表示時間を指定
	(u"（コロコロ）", u"（コロコロ）<w1.43>"),
)
	


# 一番最初にテキストを正規化するもの
NORMALIZE_TRANS_LIST=(
	# 置換の順番に注意　(変換前), (変換後)

	# 半角数字の全角化は別の方法で

	# 記号表記の修正
	(u"・・・・・・", u"……"),
	(u"・・・", u"……"),
)



#---------置き換えのための辞書型とタプル型------------------------------------#
# STRtype.translate(DICTtype) とすることで文字を置き換えることができる
# STRtype.maketrans() が python3 にしかないため、この方法を使う。
# 全角大なり・全角小なりを全角山括弧に変換
# 半角はてな・半角びっくりを全角に変換
NORMALIZE_TRANS_SYMBOL_DICT={
	ord(u"＜"): u"〈",
	ord(u"＞"): u"〉",
	ord(u"!"): u"！",
	ord(u"?"): u"？",
}
# 全角数字を半角数字に変換
NUM_ZENKAKU_TO_HANKAKU_DICT={
	ord(u"０"): u"0",
	ord(u"１"): u"1",
	ord(u"２"): u"2",
	ord(u"３"): u"3",
	ord(u"４"): u"4",
	ord(u"５"): u"5",
	ord(u"６"): u"6",
	ord(u"７"): u"7",
	ord(u"８"): u"8",
	ord(u"９"): u"9",
}
# 半角数字を全角数字に変換
NUM_HANKAKU_TO_ZENKAKU_DICT={
	ord(u"0"): u"０",
	ord(u"1"): u"１",
	ord(u"2"): u"２",
	ord(u"3"): u"３",
	ord(u"4"): u"４",
	ord(u"5"): u"５",
	ord(u"6"): u"６",
	ord(u"7"): u"７",
	ord(u"8"): u"８",
	ord(u"9"): u"９",
}
# 日本語のエンコード指定を集めたタプル型
# 参考: http://docs.python.org/library/codecs.html
# これで日本語がデコードできないなら諦める
ENCODING_LIST = ("utf-8",
	"cp932", "shift_jis", "shift-jis-2004", "shift-jisx0213",
	"utf_32", "utf_32_be", "utf_32_le",
	"utf_16", "utf_16_be", "utf_16_le",
	"utf-7", "utf-8_sig", 
	"euc-jp", "euc-jis-2004", "euc-jisx0213",
	"iso-2022-jp", "iso-2022-jp-1", "iso-2022-jp-2",
	"iso-2022-jp-2004", "iso-2022-jp-3", "iso-2022-jp-ext"
)



#---------重箱の隅的な設定：禁則処理関連の文字列の正規表現--------------------------#
# ぶら下がりの行頭禁則処理の対象文字
LINE_HEAD_HANG_RULE = u"　。、．，」）』）｝〕〉》」』】〙〗〟’”｠»" + u"\–\.\,\)\]\s\n "

# 行頭禁則処理の対象文字
LINE_HEAD_RULE = u"ヽヾーァィゥェォッャュョヮヵヶ" +\
u"ぁぃぅぇぉっゃゅょゎゕゖ" +\
u"ㇰㇱㇲㇳㇴㇵㇶㇷㇸㇹㇺㇻㇼㇽㇾㇿ" +\
u"々〻？！‐゠〜\?\!"+\
LINE_HEAD_HANG_RULE
# ぶら下げ対象文字も含む。ぶら下げ対象文字は行頭禁則文字の部分集合なので。

# 行末禁則処理の対象文字
LINE_TAIL_RULE = u"「（『（｛〔〈《「『【〘〖〝‘“｟«" + "\(\["

# 分離禁則処理の対象文字
LINE_DIVIDE_RULE = u"—…‥"

# 分離禁則処理の対象文字（英数字）
ALPHABET_NUM_DIVIDE_RULE = u"0-1a-zA-Z_\-" + u"０-９ａ-ｚＡ-Ｚ＿—"

# 語尾の読点（。）必要判定の対象文字の正規表現
# 否定の正規表現。この文字列に含まれない文字ならマッチする
GOBI_DAKUTEN_NEED = u"^、。．，！？」）』）｝〕〉》」』】〙〗〟’”｠»" + u"\.\,\)\]\n"

# はてな・びっくり後の、本来なら全角空白が必要となる対象文字の正規表現
# 否定の正規表現。この文字列に含まれない文字ならマッチする
ZENKAKU_SPACE_NEED = u"^　、。．，！？」）』）｝〕〉》」』】〙〗〟’”｠»" + u"\.\,\)\]\s\n"



#---------正規表現用の判定文字列クラス--------------------------------------------#
# コンストラクタに文字列を渡したあと
# match(is_match_single_char)の一文字がコンストラクタ文字列のどれかに一致するのか判定
class IsMatchCharInString(object):
	# どれに一致するかという意味の文字列を受け取る
	def __init__(self, string):
		self.__p = re.compile(u"[" + string + u"]")

	# 受け取った一文字がさっきの文字列のどれかに一致するか判定
	def match(self, is_match_single_char):
		m = self.__p.match(is_match_single_char)
		return bool(m)



#---------台詞抽出の親クラス--------------------------------------------#
# 台詞部分の抽出に関する関数をもつ
# クラスメソッド名をアンダーバー二本付きにすると小クラスから呼び出せなくなる
class AbstScript(object):
		
	# 台詞を抽出　取り除く記号を指定
	def _abstScriptInLine(self, abst_script, key=u""):
		# 括弧補完
		abst_script = self.__completeBracket(abst_script)
		# 抽出した文字列から不要な記号を取り除く
		abst_script = self.__deleteBracket(abst_script, key)
		# 語尾に読点が必要ならつける
		abst_script = self.__addGobiDakuten(abst_script)

		# 抽出した文字列を返す
		return abst_script
		

	# 開き括弧が閉じていないなら閉じる
	def __completeBracket(self, line):
		# 台詞部分の最初の一文字を取得
		line_first = line[:1]
		# コロンで始まるなら
		if line_first == u"：":
			# コロンの次を最初の一文字とする
			line_first = line[1:2]

		# 最初の一文字目にある丸括弧がずっと閉じていないなら
		if line_first == u"（" and line.find(u"）") < 0:
		# 語尾に閉じ丸括弧を追加
			line += u"）"
		# 最初の一文字目にある鍵括弧がずっと閉じていないなら
		elif line_first == u"「" and line.find(u"」") < 0:
		# 語尾に閉じ鍵括弧を追加
			line += u"」"

		# 台詞部分の最後の一文字を取得
		line_last = line[-1:]
		# 最後の一文字目にある丸括弧がずっと開いていないなら
		if line_last == u"）" and line.find(u"（") < 0:
		# 先頭に開き丸括弧を追加
			line = u"（" + line
		# 最後の一文字目にある丸括弧がずっと開いていないなら
		if line_last == u"」" and line.find(u"「") < 0:
		# 先頭に開き丸括弧を追加
			line = u"「" + line

		return line
	
	# 抽出した文字列から不要な記号を取り除く
	def __deleteBracket(self, abst_script, key):
		# print u"moto:" + abst_script
		# 文字列の最初と最後を取得
		abst_first = abst_script[:1]
		abst_last = abst_script[-1:]
		# 最初の一文字がコロンなら削る
		if abst_first == u"：":
			# コロン以降を取り直し
			abst_script = abst_script[1:]
			# 取り直した後で最初の一文字を取り直し
			abst_first = abst_script[:1]
			# 字幕編集用モードならそのまま返す
			if key == u"subtitle":
				return abst_script

		# 棒読みちゃん台本編集モードなら
		if key == u"speaker":
			# 台詞の最初の一文字が丸括弧かつ最期の一文字で閉じているなら
			if (abst_first == u"（") and (abst_last == u"）"):
				# 両方を取り除く
				abst_script = abst_script[1:-1]

		# 台詞の最初の一文字が鍵括弧かつ最期の一文字で閉じているなら
		if (abst_first == u"「") and (abst_last == u"」"):
			# 両方を取り除く
			abst_script = abst_script[1:-1]
		
		return abst_script

	# 文字列の語尾に読点が必要なら追加する
	def __addGobiDakuten(self, text):
		# 語尾の一文字を取得
		# 文字列が空文字だと gobi[i] で範囲外アクセスして例外を発生させてしまう
		# でもスライスだと範囲外アクセスは空文字になってくれるため安全
		gobi = text[-1:]
		# その後に読点が必要ならば
		m = IsMatchCharInString(GOBI_DAKUTEN_NEED)
		if m.match(gobi):
			# 読点を加える
			text += u"。"
		return text


#---------発言者リストの作成クラス--------------------------------------------#
# 追加された名前から役者リストを作成
# Borg パターン。要は堅くない Singleton パターン
# ざっくり言うとグローバル変数
class CastListScript(object):
	# クラス内で持つのを許されるアトリビュートを定義
	# ……したら面倒なことになるのでコメントアウト
	# __slots__ = ["__shared_state", "__lines_list", "cast_list", "cast_script"]

	# クラス変数の枠を用意
	__shared_state = {}
	# コンストラクタ
	def __init__(self):
		# クラスから生成される全インスタンスの全アトリビュートはインスタンス間で共有される
		self.__dict__ = self.__shared_state
		
		# 初めてインスタンスが生成されたならアトリビュートを用意
		if len(self.__dict__) <= 0:
			# 台本の台詞行リスト
			self.__lines_list = []
			# 発言者リスト
			self.cast_list = []
			# yukktalk 形式のキャストリスト
			self.cast_script = ""
	
	# リストを yukktalk 形式の文字列に変換
	def create(self):
		# これまでの蓄積からキャストリストを作成、取得
		cast_list = self.castList()
		# yukktalk 形式に変換
		cast_script = u"◎\n\n"
		for cast in cast_list:
			cast_script += u"※" + cast + u"=," + u"\n"
		cast_script += u"\n"
		# クラス変数に保存して返す
		self.cast_script = cast_script
		return cast_script

	# キャスト名を追加する
	def addLine(self, cast_name):
		self.__lines_list.append(cast_name)
		
	# これまで追加されたキャスト名からキャストリストを作成
	def castList(self):
		# 重複を削除
		cast_list = list( set(self.__lines_list) )

		# キャストの台詞の出現回数を数える
		cast_dic={}
		for cast in cast_list:
			cast_count = self.__lines_list.count(cast)
			# 辞書型で対応付け
			cast_dic[cast] = cast_count

		# キャストリストを出現回数の多い順にソートしてリスト追加
		cast_list = []
		for cast,count in sorted( cast_dic.items(), key=lambda x:x[1], reverse=True):
			# ぶっちゃけ↑のラムダ式はよくわからんかった
			# 理解はしたけどそらで書ける気がしない
			cast_list.append(cast)

		# クラス変数に保存して返す
		self.__cast_list = cast_list
		return cast_list



#---------発言者指定部分の変換クラス--------------------------------------------#
# インスタンスが生成された後、文字列を待ち、文字列を受け取ると変換
# 変換後の文字列はクラス変数に持ち続ける
class ToSpeakerCommand(object):
	# クラス内で持つのを許されるアトリビュートを定義
	__slots__ = ["speaker_command", "castlist"]
	# コンストラクタ
	def __init__(self):
		self.speaker_command = u""
		self.castlist = CastListScript()

	def translation(self, line, name_end):
		# 名前,コマンド「台詞」
		# という状態で受け取った一行から名前とコマンドを一緒に取得
		name_with_command = line[:name_end]
		# 名前を取得
		order_name = self.__splitOrderName(name_with_command)
		self.castlist.addLine(order_name)
		# 指示を取得
		order_command = self.__splitOrderCommand(name_with_command)

		# yukktalk 書式に変換して内容をクラス変数に保持
		speaker_command = u"◆" + order_name + order_command + u"\n"
		self.speaker_command = speaker_command
		# 変換後の文字列を返す
		return speaker_command
	
	def __splitOrderName(self, name_with_command):
		# コマンド開始位置を調べて
		command_start = name_with_command.find(u",")
		if command_start < 0:
			# コンマがないならコマンド無しとして扱う
			command_start = None
		# その手前にある名前を抜き出す
		order_name = name_with_command[:command_start]

		# 名前が括弧付きだった場合の処理を行う
		order_name = self.__handleForBracket(order_name)
		return order_name
	
	# 名前が括弧付きだった場合はどう処理するか　設定にしたがって行う
	def __handleForBracket(self, order_name):
		# 名前部分の括弧を調べる
		bracket_first = order_name.find(u"（")

		# 括弧関係なしでそのままにするなら
		if bracket_first < 0 or SPEAKER_COMMAND_WITH_INSIDE_BRACKET:
			bracket_last = order_name.find(u"）")
			# もし丸括弧が開いたまま閉じていないなら閉じておく
			if bracket_first >= 0 and bracket_last < 0:
				order_name += u"）"

		# 括弧に対して処理をするなら
		else:
			if NAME_IN_BRACKET_OUTSIDE_OR_INSIDE == u"INSIDE":
				# 括弧の中身を取り出す
				bracket_last = order_name.find(u"）")
				if bracket_last < 0:
					bracket_last = None
				order_name = order_name[bracket_first+1 : bracket_last]
			else:
				# 括弧の外側、つまり括弧より手前を取り出す
				order_name = order_name[:bracket_first]
		return order_name

	def __splitOrderCommand(self, name_with_command):
		# コマンド開始位置を調べて
		command_start = name_with_command.find(u",")
		if command_start >=0:
			# コンマ以降の指示を抜き出す
			order_command = name_with_command[command_start:]
		else:
			# コンマがないならコマンド無しとして扱う
			order_command = u""

		# 全角数字を半角数字に正規化してから返す
		order_command = order_command.translate(NUM_ZENKAKU_TO_HANKAKU_DICT)
		return order_command



#---------禁則処理クラス-------------------------------------#
# インスタンスが生成された後、文字列と改行場所を待ち、受け取るとその改行場所が適切か計算
# 計算後は改行場所の相対距離を保持
class LineBreakingRule(object):
	# クラス内で持つのを許されるアトリビュートを定義
	__slots__ = ["i"]

	# コンストラクタ
	def __init__(self):
		# 折り返しごとに格納するためのリストを定義
		self.i = 0
	
	# 禁則処理
	def applyingResult(self, text, index_now):
		# 行末、次行頭、次行頭の次の文字を調べる
		i=index_now
		line_last = text[i-1:i]
		nextline_first = text[i:i+1]
		nextline_second = text[i+1:i+2]

		# 禁則処理開始　順番に注意
		# 行頭禁則処理が必要なら
		if self.__isNeedLineBreakingRule(nextline_first, LINE_HEAD_RULE):
			# ぶら下げ対象かつ
			# ぶら下げた結果が行頭禁則にならないのであれば
			headhang_bool = \
				self.__isNeedLineBreakingRule(nextline_first, LINE_HEAD_HANG_RULE)
			headpush_bool = not \
				self.__isNeedLineBreakingRule(nextline_second, LINE_HEAD_RULE)
			if headhang_bool and headpush_bool:
				# ぶら下げ
				i+= 1
			# ぶら下げないなら押し出し
			else:
				i-=1
		# 分離禁則処理が必要なら
		elif self.__isNeedLineBreakingRule(line_last, LINE_DIVIDE_RULE):
			# 対象の文字が分離していると分かったら
			if line_last == nextline_first:
				# 押し出し
				i-=1 
		# 英数字の分離禁則処理が必要そうなら
		elif self.__isNeedLineBreakingRule(line_last, ALPHABET_NUM_DIVIDE_RULE):
			# 次行の先頭も英数字だったら
			if self.__isNeedLineBreakingRule(nextline_first, ALPHABET_NUM_DIVIDE_RULE):
				# 押し出し
				i-=1
		# 行末禁則処理が必要なら
		elif self.__isNeedLineBreakingRule(line_last, LINE_TAIL_RULE):
			# 押し出し
			i-=1 
	
		# もし禁則処理の結果が0以下なのであればインデックスを元に戻す
		if i <0:
			i= index_now
		# もし禁則処理を行ったのであれば、
		# そして強い禁則処理を行うのであれば、もう一度行う
		elif i != index_now and STRICT_LINE_BREAKING_RULE:
			i += self.applyingResult(text, i)

		# 最初の位置からの差分を、クラス変数に保持してから返す
		self.i = i-index_now
		return i-index_now
		
	# 禁則処理の対象文字かか否か
	def __isNeedLineBreakingRule(self, char, string):
		m = IsMatchCharInString(string)
		return m.match(char)



#---------文章の折り返し作業クラス-------------------------------------#
# インスタンスが生成された後、文字列を待ち、文字列を受け取ると変換
# 変換後の文字列はクラス変数に持ち続ける
class DevideSubtitleForLines(object):
	# クラス内で持つのを許されるアトリビュートを定義
	__slots__ = ["decosub_script", "__rule"]

	# コンストラクタ
	def __init__(self):
		self.__rule = LineBreakingRule()
		# 折り返しごとに格納するためのリストを定義
		self.decosub_script = u""

	# 文字列に改行を挿入していく
	def translation(self, speak_script):
		decosub_script = speak_script
		# インデックスリセット
		i=0
		while len(decosub_script)-i > 0:
			# インデックスを進める前に最初の位置を記憶
			line_begin = i
			# 一行分インデックスを進める
			i += self.__ptrForwardOneLine(decosub_script, i)
			# インデックスを進めた後の位置を記憶
			line_end = i
			
			# 禁則処理をするなら
			if USE_LINE_BREAKING_RULE == True:
				# 禁則処理
				i += self.__rule.applyingResult(decosub_script, i)
				# 禁則処理の結果インデックスが一行分より多く戻るなら禁則処理は行わない
				if i <= line_begin:
					i = line_end

			# 進めた位置が文字列の終端じゃないなら
			if decosub_script[i:i+1] != u"":
				# 進めた位置に改行を挿入
				decosub_script = decosub_script[:i] + u"\n" + decosub_script[i:]
				# 改行分、インデックスをすすめる
				i += 1

		# 文字列をクラス変数に保持してから返す
		self.decosub_script = decosub_script
		return decosub_script
		
	def __ptrForwardOneLine(self, decosub_script, index_start):
		# 字幕内の（コロコロ）のインデックスを取得
		koro = u"（コロコロ）"
		start_koro = decosub_script.find(koro)
	
		i=index_start
		# もし行頭よりも先かつ文字幅以内に（コロコロ）があるなら
		if i < start_koro and i+MAX_LINE_LENGTH > start_koro:
			# （コロコロ）の手前までインデックスを進める
			i=start_koro
		# ないなら
		else:
			# 折り返しの文字幅分インデックスを進める
			i += MAX_LINE_LENGTH
	
		# インデックスをどれだけ進めたか返す
		return i-index_start



#---------AviUtl字幕表示用の制御文字などの変換クラス-------------------------#
# インスタンスが生成された後、文字列を待ち、文字列を受け取ると変換
# 変換後の文字列はクラス変数に持ち続ける
class ToScriptSubtitleLinesList(object):
	# クラス内で持つのを許されるアトリビュートを定義
	__slots__ = ["subtitle_lines_list", "__devide_subtitle"]

	# コンストラクタ
	def __init__(self):
		self.__devide_subtitle = DevideSubtitleForLines()
		# 折り返しごとに格納するためのリストを定義
		self.subtitle_lines_list = []

	def translation(self, speaker_name, speaker_script):
		# 折り返しごとに格納するためのリストを準備
		subtitle_lines_list =[]

		# まず発言者表示から
		# 発言者表示をするのなら
		if SHOW_NAME_TO_SUBTITLE == True:
			# 発言者の名前表示位置を調節する
			# decosub = decorated subtitle
			decosub_name = self.__speakerNameDecoration(speaker_name)
			# 発言内容を改行で区切ってリスト分け
			decosub_name = decosub_name.split("\n")
			# 分けたものを順番にリストへ追加
			for line in decosub_name:
				subtitle_lines_list.append(line)
		
		# 次は発言内容の字幕
		# 指定の文字数ごとに改行を挿入
		# decosub = decorated subtitle
		decosub_script = self.__devide_subtitle.translation(speaker_script)

		# 改行で区切ってリスト分け
		decosub_script = decosub_script.split("\n")
		# 分けたものを順番にリストへ追加
		for line in decosub_script:
			# 一行ごとに字幕制御文字を挿入
			line = self.__textToSubtitleCtrl(line)
			# 一行ごとにリストへ追加
			subtitle_lines_list.append(line)

		# リストをクラス変数に保持してから返す
		self.subtitle_lines_list = subtitle_lines_list
		return subtitle_lines_list
	
	# AviUtl での字幕制御文字で名前の表示を装飾する
	def __speakerNameDecoration(self, speaker_name):
		# 名前表示の前に全角空白を入れる
		decosub_name = u"　"*ZENKAKU_SPASE_BEFORE_NAME + speaker_name
		# もし名前表示のフォントサイズ指定があれば
		if NAME_SIZE > 0:
			# AviUtl 拡張編集ののテキスト制御文字で囲う
			decosub_name = u"<s" + str(NAME_SIZE) + u">" + decosub_name + u"<s>"
		# 名前の後に空改行を加える
		decosub_name += u"\n"*NEWLINE_AFTER_NAME
		return decosub_name
			
	# 字幕に Aviutl 用字幕表示制御文字を入れていく
	def __textToSubtitleCtrl(self, text):
		for before, after in SUBTITLE_CTRL_TRANS_LIST:
			text = text.replace(before, after)
		return text



#---------AviUtl字幕用台本部分の変換クラス-------------------------------------#
# インスタンスが生成された後、文字列を待ち、文字列を受け取ると変換
# 変換後の文字列はクラス変数に持ち続ける
class ToScriptSubtitle(AbstScript):
	# クラス内で持つのを許されるアトリビュートを定義
	__slots__ = ["subtitle_script", "__sub_list"]

	# コンストラクタ
	def __init__(self):
		self.__sub_list = ToScriptSubtitleLinesList()
		# 変換後の文字列
		self.subtitle_script = u""

	# AviUtl 字幕用に台本を変換
	def translation(self, line, name_end):
		# 名前,コマンド「台詞」
		# という状態で受け取った一行から名前とコマンドを一緒に取得
		name_with_command = line[:name_end]
		# 名前を取得
		order_name = self.__splitOrderName(name_with_command)

		# 台詞部分をとにかく抽出
		abst_script = line[name_end:]
		# 頭と語尾から括弧を取り除く
		speaker_script = AbstScript.\
			_abstScriptInLine(self, abst_script, key=u"subtitle")

		# 字幕表示一行ごと折り返すためにリスト分け
		subtitle_lines_list = self.__sub_list.translation(order_name, speaker_script)

		# リスト分けしたものを一行ずつ yukktalk 書式に変換
		subtitle_script =u""
		for temp_line in subtitle_lines_list:
			# yukktalk 書式に変換
			subtitle_script += u"[字幕]" + temp_line + u"\n"
		
		# 変換後の文字列をクラス変数に保持してから返す
		self.subtitle_script = subtitle_script
		return subtitle_script

	# 名前とコマンドが一緒になっている文字列から名前を抜き出す
	def __splitOrderName(self, name_with_command):
		# コマンド開始位置を調べて
		command_start = name_with_command.find(u",")
		if command_start < 0:
			# コンマがないならコマンド無しとして扱う
			command_start = None
		# その手前にある名前を抜き出す
		order_name = name_with_command[:command_start]

		# 発言者名の開き括弧が閉じていないなら
		if order_name.find(u"（") >= 0 and order_name.find(u"）") < 0:
			# 閉じる
			order_name += u"）"
		return order_name

#---------日本語の読み解析クラス-------------------------------------#
# 日本語の文字列を入力すると、読みのみの文字列にして返す
MECAB_ENCODING_LIST = ("euc-jp", "utf-8", "shift_jis")
class StringToReadable(object):
	# クラス内で持つのを許されるアトリビュートを定義
	__slots__ = ["concise_words", "__tagger"]

	# コンストラクタ
	def __init__(self):
		# 変換後の文字列
		self.concise_words = u""
		# MeCab を茶筅モードで起動
		if isMeCab:
			self.__tagger = MeCab.Tagger("-Ochasen")
		else:
			self.__tagger = None

	# 読みを解析
	def translation(self, string):

		# MeCab を使って解析。結果がリストで返ってくる
		mecab_result = self.__useMeCab(string)
		
		# MeCab を使って解析した読み方を整理する
		concise_words = self.__mergeMeCabResult(mecab_result)
		
		# クラス変数に保持して返す
		self.concise_words = concise_words
		return concise_words

	# カタカナをひらがなに変換
	# ここ参考にした
	# http://d.hatena.ne.jp/mohayonao/20101213/1292237816
	def __hiragana(self, text):
		"""ひらがな変換"""
		re_katakana = re.compile(ur'[ァ-ヴ]')
		return re_katakana.sub(lambda x: unichr(ord(x.group(0)) - 0x60), text)
	
	
	# ひらがなをカタカナに変換
	def __katakana(self, text):
		"""カタカナ変換"""
		re_hiragana = re.compile(ur'[ぁ-ゔ]')
		return re_hiragana.sub(lambda x: unichr(ord(x.group(0)) + 0x60), text)
	
	# MeCab を使って解析
	# ここでは str 型の文字列しか扱わない
	def __useMeCab(self, string):
		# 解析前に unicode 型から str 型に変換する
		# MeCab とのやりとりは unicode 型ではなく str 型でないとうまくいかないため
		# 解析木の準備
		node = self.__tagger.parseToNode( str(string) )

		mecab_result = []
		# C++ の双方向リストをたどっていく
		while node:
			# 品詞ID を取得
			id_no = node.id
			# print id_no
			# 品詞の元の文字列を取得
			surface = unicode( node.surface[:] )

			# 解析結果を取得
			feature_list = unicode( node.feature[:] )
			# csv 形式なのでカンマで区切ってリストにする
			feature_list = feature_list.split(u",")

			# 結果をリストのリストに追加
			mecab_result.append( [feature_list, surface, id_no] )

			# 解析木の次の要素へ
			node = node.next

		return mecab_result

	def __mergeMeCabResult(self, mecab_result):
		concise_words = u""
		# 読み解析結果と元の品詞をリストから順番に取り出す
		for feature_list, surface, id_no in mecab_result:
			# 本当はハードコーディングでダメだけど
			# 9番目の要素が読みなので取り出す
			try:
				feature = feature_list[8]
			# 「読み」の項目が空なら
			except IndexError:
				# 空にする
				feature = None

			# もし解析結果が空なら元の品詞を受け取って次の品詞へ
			if feature == None or feature == "*":
				concise_words += surface
				continue

			# 解析対象がひらがなのみなら
			m = re.compile(u"^[あ-ゔ]+$")
			if m.match(surface):
				# 受け取って次の品詞へ
				concise_words += self.__mergeHiragana(feature, surface)
				continue
	
			# id_no がうまくいかないので解析結果リストから判定
			# 本当はハードコーディングでダメだけど
			# 解析対象が助数詞 (id_no=53) か数 (in_no=48) なら
			if feature_list[1] == u"数" or feature_list[2] == u"助数詞":
				# そのまま受け取って次の品詞へ
				concise_words += surface
				continue

			# 副詞ならひらがなにする。記者ハンドブックによるとこの方が読みやすいらしい
			if feature_list[0] == u"副詞":
				# ひらがなにして受け取って次の品詞へ
				concise_words += self.__hiragana(feature)
				continue
				
			# 解析対象を置換するなら
			temp = surface
			for before, after in MECAB_DIVIDE_REPLACE:
				# str.replace(src, dst) を使うと熟語の片割れまで置換してしまうので
				# 品詞分解後の単語の完全一致でのみ置換を行う
				if temp == before:
					temp = after
			# もし置換があったならば
			if temp != surface:
				# 置換後の文字列をとって次の品詞へ
				concise_words += temp
				continue
			
			# 解析対象が動詞なら
			if feature_list[0] == u"動詞":
				# そのまま受け取って次の品詞へ
				concise_words += surface
				continue

			# 解析対象がひらがなを含まず漢字のみなら
			m = re.compile(u"^[^あ-ゔ]+$")
			if m.match(surface):
				# 読みをカタカナで受け取って次の品詞へ
				concise_words += feature
				continue
			
			# どの条件も満たしていないなら元の文字のまま
			concise_words += surface
			# 解析結果の読みと送り仮名を合わせる
			# concise_words += self.__mergeOkurigana(feature, surface)
		# 結果を返す
		return concise_words
	
	# 解析対象がひらがなのみなら
	def __mergeHiragana(self, feature, surface):
		# 読みを平仮名に変換
		feature = self.__hiragana(feature)
		
		# 「は」の読みが「わ」なら
		ha = surface.find(u"は")
		if ha >= 0 and feature[ha] != surface[ha]:
			feature = feature[:ha] + u"わ" + feature[ha+1:]
		
		# 「へ」の読みが「え」なら
		he = surface.find(u"へ")
		if he >= 0 and feature[he] != surface[he]:
			feature = feature[:he] + u"え" + feature[he+1:]

		return feature


	# 読み解析結果と元の品詞を受け取ると、送り仮名を平仮名にして返す
	def __mergeOkurigana(self, feature, surface):
		# 元の文字列を保存
		concise_words = feature
		# 読みを平仮名に変換
		feature = self.__hiragana(feature)

		# 頭が一致するかどうか
		if feature[:1] == surface[:1]:
			# もし一致したなら、一致しなくなるまで先頭から文字数を数える
			for i in range( len(surface) ):
				if feature[i] != surface[i]:
					break
			# 一致した文字だけひらがなのままでマージ
			concise_words = surface[:i] + concise_words[i:]
		
		# 語尾が一致するかどうか
		if feature[-1:] == surface[-1:]:
			# もし一致したなら、一致しなくなるまで語尾から文字数を数える
			# イテレートされるリストは [-1, -2, -3..len(surface)]
			for i in range(-len(surface), 0)[::-1]:
				if feature[i] != surface[i]:
					break
			# 一致した文字だけひらがなのままでマージ
			concise_words = concise_words[:i+1] + surface[i+1:]

		# マージした文字列を返す
		return concise_words
		


#---------棒読みちゃん用台本部分の変換クラス-------------------------------------#
# インスタンスが生成された後、文字列を待ち、文字列を受け取ると変換
# 変換後の文字列はクラス変数に持ち続ける
class ToBoyomiScript(AbstScript):
	# クラス内で持つのを許されるアトリビュートを定義
	__slots__ = ["boyomi_script", "readable"]

	# コンストラクタ
	def __init__(self):
		self.readable = StringToReadable()
		# 変換後の文字列
		self.boyomi_script = u""
		
	# 棒読みちゃん用台本に変換
	def translation(self, line, name_end):
		# 台詞部分をとにかく抽出
		abst_script = line[name_end:]
		# 頭と語尾からいらない括弧を取り除く
		boyomi_script = AbstScript.\
			_abstScriptInLine(self, abst_script, key=u"speaker")

		# 棒読みちゃんお助け変換を行う設定なら
		if USE_BOYOMICHAN_HELP:
			# 棒読みちゃん用にふりがな
			boyomi_script = self.__textToBoyomiHelp(boyomi_script)
		
		# MeCab を使って読み解析を行うなら
		if USE_MECAB and isMeCab:
			boyomi_script = self.readable.translation(boyomi_script)
			# 「は」を確実に「は」と発音させるための変換
			if TRANS_HA:
				boyomi_script = boyomi_script.replace(u"は", u"ha")
		# 語尾から読点「。」を取り去る設定なら取り去る
		if DELETE_GOBI_DAKUTEN:
			boyomi_script = boyomi_script.rstrip(u"。")

		# 台詞を yukktalk 書式に変換して内容をクラス変数に保持
		boyomi_script = u"[音声]"+ boyomi_script +u"\n"
		self.boyomi_script = boyomi_script
		# 変換後の文字列を返す
		return boyomi_script

	# 棒読みちゃんが読み間違えないために文字列を置換
	def __textToBoyomiHelp(self, boyomi_script):
		# 棒読みちゃん用置換リストで置換
		for before, after in BOYOMICHAN_TRANS_LIST:
			boyomi_script = boyomi_script.replace(before, after)

		# 全角数字を半角数字に変換
		boyomi_script = boyomi_script.translate(NUM_ZENKAKU_TO_HANKAKU_DICT)
		return boyomi_script



#---------yukktalk_script用変換クラス--------------------------------------------#
# インスタンスが生成された後、文字列を待ち、文字列を受け取ると変換
# 変換後の文字列はクラス変数に持ち続ける
class YukktalkScript(object):
	# クラス内で持つのを許されるアトリビュートを定義
	__slots__ = ["translated_text",\
		"__speaker_command", "__subtitle_script", "__boyomi_script"]

	# コンストラクタでアトリビュートを初期化
	def __init__(self):
		self.__speaker_command = ToSpeakerCommand()
		self.__subtitle_script = ToScriptSubtitle()
		self.__boyomi_script = ToBoyomiScript()
		# yukktalk 用の台本テキストに変換した後のテキスト
		self.translated_text = u""

	# yukktalk 用台本テキストに変換する
	def translation(self, src_text):
		# いくつかの下準備
		# 最初に置換したい文字列を置換して正規化
		line = self.__textToNormalize(src_text)
		# 行内の名前部分の位置を取得
		name_end = self.__nameIndex(line)
		# 変換後のテキスト
		translated_text = u""

		# この行に名前と台詞があるなら
		if name_end >= 0:
			
			# 発言者指定の書式を変換
			translated_text += self.__speaker_command.translation(line, name_end)
			# 発言時の字幕文章を変換
			translated_text += self.__subtitle_script.translation(line, name_end)
			# 発言時の読み上げ台本を変換
			translated_text += self.__boyomi_script.translation(line, name_end)

		# 台詞じゃない行なら
		else:
			# 文字列をそのままスルーして出力
			translated_text += line + u"\n"

		# 変換した文字列をクラス変数に保持してから返す
		self.translated_text = translated_text
		return translated_text

	# 台本の一行を受け取って名前の位置を調べる
	# line[:name_end] で名前部分をスライスできるようにインデックスを返す
	def __nameIndex(self, line):
		index_list = []
		# 鍵括弧とコロンと丸括弧の位置をとる
		# インデックスから記号が存在するか調べる
		# 鍵括弧
		kagikakko_index = line.find(u"「")
		index_list.append(kagikakko_index)
		# コロン
		colon_index = line.find(u"：")
		index_list.append(colon_index)

		# 丸括弧
		# 二つあるなら二つ目をとってほかの括弧と比較
		# 一つだけなら　ほかの括弧がなければ確定。それのみ操作。
		# ほかの括弧があるなら、閉じ括弧の位置を調べて、とじよりも右にほかの括弧があるならそっちを優先
		marukakko_index = line.find(u"（")
		# 開き丸括弧がもし二つ以上あるなら
		if line.count(u"（") > 1:
			# 二つ目の開き丸括弧の位置をとる
			marukakko_index = line.find(u"（", marukakko_index+1 )

		# 開き丸括弧がひとつで、
		elif line.count(u"（") == 1:
			tojimaru_index = line.find(u"）")
			# 閉じ括弧がないなら
			if tojimaru_index < 0:
				# 他の括弧が存在しているなら
				if max(index_list) >= 0:
					# 丸括弧は存在していないものとみなす
					marukakko_index = -1
			# 行の途中で閉じているなら
			elif tojimaru_index < len(line)-1:
				# 他の括弧が存在しているなら
				if max(index_list) >= 0:
					# 丸括弧は存在していないものとみなす
					marukakko_index = -1
				# 他の括弧が存在していないなら
				else:
					# 閉じ括弧の次の文字から台詞が始まるとする
					marukakko_index = tojimaru_index+1

		# 丸括弧
		index_list.append(marukakko_index)

		# 各記号のインデックスを比較
		name_end = LONGEST_NAME_LENGTH
		for nominee in index_list:
			# 0以上かつこれまでのインデックスより左にあれば
			if nominee >= 0 and nominee < name_end:
				# インデックスの値を更新
				name_end = nominee
			
		# 行頭 LONGEST_NAME_LENGTH 文字以内に名前が無いなら操作対象とみなさない
		# この判定方法はかなり曖昧
		if name_end >= LONGEST_NAME_LENGTH:
			name_end = -1

		# 名前終わりのインデックスを返す
		return name_end
	
	# 最初に文字列を正規化する
	def __textToNormalize(self, text):
		# NORMALIZE_TRANS_LIST から置換を行う
		for before, after in NORMALIZE_TRANS_LIST:
			text = text.replace(before, after)

		# 半角数字を全角数字に変換
		text = text.translate(NUM_HANKAKU_TO_ZENKAKU_DICT)
		# 全角大なり・全角小なりを全角山括弧に変換
		# 半角はてな・半角びっくりを全角に変換
		text = text.translate(NORMALIZE_TRANS_SYMBOL_DICT)

		# はてなのあとの空白を挿入
		# 文字列を最後の一字手前までなめる
		for i in range(len(text)-1):
			# 全角の『？・！』を見つけた時
			now_char = text[i]
			if now_char == u"？" or now_char == u"！":
				# どれかに一致という条件設定
				# その次の文字に本来なら全角空白があるべきところなら
				next_index = i+1
				next_char = text[next_index]
				m = IsMatchCharInString(ZENKAKU_SPACE_NEED)
				if m.match(next_char):
					# つぎの文字に全角空白を挿入する
					text = text[:next_index] + u"　" + text[next_index:]
		# 正規化後の文字列を返す
		return text



#---------テキストファイル作成クラス--------------------------------------------#
# 
# 渡されたファイル名を元に、新しくつくるファイル名を決める
class FixNewFilename(object):
	# クラス内で持つのを許されるアトリビュートを定義
	__slots__ = ["new_filename"]

	# コンストラクタでアトリビュートを初期化
	# 変換元のファイル名は省略可能
	def __init__(self):
		# アトリビュートの初期化
		# 新しくつくるファイル名
		self.new_filename = ""
	
	# ほかと重ならずに作れるファイル名を探す
	def search(self, src_filename="yukktalk_transleted.txt"):
		# 新しく作るファイル名につけるための日時を取得
		d = datetime.datetime.today()
		timestamp = "_" + str(d.year) + str(d.month).zfill(2)+ str(d.day).zfill(2)

		# 新しく作るファイル名が重ならないかチェック
		temp_name = "-yukk" + timestamp + ".txt"
		new_filename = src_filename.replace(".txt", temp_name)
		i=0
		# もし重なるなら
		while os.path.isfile(new_filename) == True :
			# ファイル末尾に数字をつける
			i+=1
			temp = "-yukk" + timestamp + "-"+ str(i) +".txt"
			new_filename = src_filename.replace(".txt", temp)

		# 新しいファイル名をクラス変数に保持して終了
		self.new_filename = new_filename
		return new_filename



#---------テキストファイル読み込みクラス----------------------------------------#
# ファイル名を読み込むと日本語テキストのエンコーディングを調べる
class SearchTextEncoding():
	# クラス内で持つのを許されるアトリビュートを定義
	__slots__ = ["src_filename", "encoding_style"]

	# コンストラクタでアトリビュートを初期化
	def __init__(self):
		# アトリビュートの初期化
		# 開くテキストファイル名
		self.src_filename = ""
		# 開くテキストファイルのエンコーディング
		self.encoding_style = ""
	
	# ファイルを調べる
	def search(self, argv):
		# コマンドライン引数でファイル名を指定されたならそっちを優先
		if len(argv)>=2:
			src_filename = argv[1]
		else:
			src_filename = FILE
		# もし指定されたファイルが存在しなければ終了
		if os.path.isfile(src_filename) == False :
			print("File Not Found!")
			sys.exit()

		# テキストエンコーディングを調べる
		encoding_style = self.__tryDecodeJapanese(src_filename)

		# 開いたファイル名と文字コード名をクラス変数に保持
		self.encoding_style = encoding_style
		self.src_filename = src_filename
		# 返す
		return encoding_style, src_filename

	# テキストが開けるかどうか文字コードごとに試行
	def __tryDecodeJapanese(self, src_filename):
		encoding_style = ""
		# エンコーディングリストから
		for encoding_style in ENCODING_LIST:
			# テキストを開く
			f = codecs.open(src_filename, "r", encoding_style)
			try:
				# テキストを先頭から末尾まで一端全部デコードしてみる
				for line in f:
					src_text = line
			# デコードエラーが出たら
			except (UnicodeDecodeError, UnicodeError):
				# エンコーディングリストから次のエンコーディングを試す
				continue
			# デコードに成功したら例外を送出せずループ脱出
			break

		# ループ脱出でファイルを閉じる
		f.close()

		# もしエンコードリストがすべてダメなら
		try:
			# src_text が未定義で例外になるはず
			src_text += ""
		
		# src_text が未定義で例外ならデコードに失敗しているのでプログラム終了
		except UnboundLocalError:
			print("File Open Error! Not Japanese Text!")
			sys.exit()

		# 読み込みに成功したエンコーディングを返す
		return encoding_style



#---------メイン関数--------------------------------------------#
# エントリポイント：ここから始まる
if __name__ == "__main__":
	# 処理に必要なクラスを初期化
	# テキストを開くためのクラス
	text_codec = SearchTextEncoding()
	# 新しくつくるテキストのファイル名を調べるクラス
	fnf = FixNewFilename()
	# 読み込んだテキストを yukktalk 形式に書き出すためのクラス
	script = YukktalkScript()
	# 読み込んだテキストからキャストリストを生成するためのクラス
	castlist = CastListScript()
	
	
	# 開始メッセージ
	print("START!")

	# 指定されたテキストファイルの、存在とコーデックを調べる
	encoding_style, src_filename = text_codec.search(sys.argv)
	f = codecs.open(src_filename, "r", encoding_style)

	# 新しくつくるファイル名を決める
	new_filename = fnf.search(src_filename)
	nf = open(new_filename, "a")

	# テキスト一行ごとにイテレートして処理して書き出し
	body = u""
	for line in f:
		# イテレートした文字列の末尾には改行コードがあるので削除
		line = line.rstrip()
		# 一行を処理
		body += script.translation(line)

	# yukktalk 用のヘッダを作成
	header = castlist.create()

	# 結果を新しいファイルに書き込み
	nf.write(header)
	nf.write(body)
	

	# 変換元のテキストと書き出したテキストを閉じる
	f.close()
	nf.close()
	
	# 終了メッセージ
	print("Finish!")
